/* Adapted from:
 * D. J. Bernstein
 * Public domain.
 */

#include "verasco.h"

#define X_12 \
  X(0);X(1);X(2);X(3); X(4);X(5);X(6);X(7); X(8);X(9);X(10);X(11)

typedef int uint32;

static uint32 seed[32] = { 3,1,4,1,5,9,2,6,5,3,5,8,9,7,9,3,2,3,8,4,6,2,6,4,3,3,8,3,2,7,9,5 } ;
static uint32 in[12];
static uint32 out[8];
static int outleft = 0;

#define ROTATE(x,b) (((x) << (b)) | ((x) >> (32 - (b))))
#define MUSH(i,b) x = t[i] += (((x ^ seed[i]) + sum) ^ ROTATE(x,b));

static void surf(void)
{
  uint32 t[12]; uint32 x; uint32 sum = 0;
  int r; int i; int loop;

  // initialization of t
#define X(i) \
  t[i] = in[i] ^ seed[12 + i]
  X_12;
#undef X
  for (i = 0;i < 8;++i) out[i] = seed[24 + i];
  x = t[11];
  for (loop = 0;loop < 2;++loop) {
    for (r = 0;r < 16;++r) {
      sum += 0x9e3779b9;
      MUSH(0,5) MUSH(1,7) MUSH(2,9) MUSH(3,13)
      MUSH(4,5) MUSH(5,7) MUSH(6,9) MUSH(7,13)
      MUSH(8,5) MUSH(9,7) MUSH(10,9) MUSH(11,13)
    }
    for (i = 0;i < 8;++i) out[i] ^= t[i + 4];
  }
}

void randombytes(unsigned char *x,unsigned long long xlen)
{
  unsigned long long i;
  for( i = 0; i < xlen; ++i) {
    if (!outleft) {
      if (!++in[0]) if (!++in[1]) if (!++in[2]) ++in[3];
      surf();
      outleft = 8;
    }
    x[i] = out[--outleft];
  }
}

#define N 0x10
unsigned char m[N];

int
main(void)
{
  randombytes(m, N);

  int i = verasco_any_int();
  verasco_assume( 0 <= i );
  unsigned d = m[i % N];
  verasco_assert( d < 0x100 );
  return 0;
}

