(* *********************************************************************)
(*                                                                     *)
(*              The Verasco static analyzer                            *)
(*                                                                     *)
(*          Vincent Laporte, Université de Rennes 1                    *)
(*          Jacques-Henri Jourdan, INRIA Paris-Rocquencourt            *)
(*                                                                     *)
(*  Copyright Institut National de Recherche en Informatique et en     *)
(*  Automatique and Université de Rennes 1.  All rights reserved.      *)
(*  This file is distributed under the terms of the GNU General        *)
(*  Public License as published by the Free Software Foundation,       *)
(*  either version 2 of the License, or (at your option) any later     *)
(*  version.  This file is also distributed under the terms of the     *)
(*  INRIA Non-Commercial License Agreement.                            *)
(*                                                                     *)
(* *********************************************************************)

Require Import CsharpminorAnalysis CsharpminorIterproof MemCsharpminorproof.
Import
  Utf8
  Coqlib Behaviors Csharpminor
  DomKind
  Util AbMemSignatureCsharpminor DebugAbMemCsharpminor.

Section VA.

  Context (kind: num_dom_kind)
          (max_concretize: N)
          (trace verbose: bool)
          (unroll: nat).

  Let vanalysis := vanalysis kind max_concretize trace verbose unroll.

  Theorem vanalysis_correct :
    forall prog, vanalysis prog = (tt, nil) ->
    forall tr, program_behaves (semantics prog) (Goes_wrong tr) ->
    False.
  Proof.
    unfold vanalysis, doit. intros prog.
    destruct kind, trace, verbose;
    intros; eapply iter_program_sound; eauto;
    match goal with
      | |- mem_dom_spec _ (debug_mem_dom _ _) _ _ => eapply debug_mem_dom_sound
      | _ => idtac
    end;
    eapply mem_cshm_dom_correct.
  Qed.

  Local Notation "⟦ p ⟧" := (program_behaves (semantics p)).

  Corollary vanalysis_sound:
    ∀ p, vanalysis p = (tt, nil) →
         ⟦p⟧ ⊆ not_wrong.
  Proof.
    intros p H (); try easy.
    intros t H0. exact (vanalysis_correct p H _ H0).
  Qed.

End VA.
