(* *********************************************************************)
(*                                                                     *)
(*              The Verasco static analyzer                            *)
(*                                                                     *)
(*          Vincent Laporte, Université de Rennes 1                    *)
(*                                                                     *)
(*  Copyright Institut National de Recherche en Informatique et en     *)
(*  Automatique and Université de Rennes 1.  All rights reserved.      *)
(*  This file is distributed under the terms of the GNU General        *)
(*  Public License as published by the Free Software Foundation,       *)
(*  either version 2 of the License, or (at your option) any later     *)
(*  version.  This file is also distributed under the terms of the     *)
(*  INRIA Non-Commercial License Agreement.                            *)
(*                                                                     *)
(* *********************************************************************)

open Printf
open Errors
open Compiler
open Camlcoq

(* Printing of error messages *)

(* Locally named idents are encoded as in Cminorgen.
   This function should live somewhere more central. *)
let ident_name = function
  | P.Coq_xI n -> Printf.sprintf "$%ld" (Camlcoq.P.to_int32 n)
  | P.Coq_xO n -> Camlcoq.extern_atom n
  | P.Coq_xH -> "$0"

let print_error oc msg =
  let print_one_error = function
  | Errors.MSG s -> output_string oc (Camlcoq.camlstring_of_coqstring s)
  | Errors.CTX i -> output_string oc (Camlcoq.extern_atom i)
  | Errors.POS i -> fprintf oc "%ld" (Camlcoq.P.to_int32 i)
  in
    List.iter print_one_error msg;
    output_char oc '\n'

let do_res (r : 'a res) : 'a = 
  match r with
    | OK v -> v
    | Error msg ->
      print_error stderr msg;
      exit 2
