open Camlcoq

let charListTr: char list -> string
= fun l ->
(*
	let s = String.create (List.length l) in
	let _ = List.fold_left (fun i c -> String.set s i c; i + 1) 0 l in
	s
*)
  let buf = Buffer.create (List.length l) in
  List.iter (fun c -> Buffer.add_char buf c) l;
  Buffer.contents buf;;

let stringTr: string -> char list
= fun s ->
	let rec f: int -> char list -> char list
	= fun i l ->
		if i >= 0
		then f (i - 1) ((String.get s i)::l)
		else l
	in
	f ((String.length s) - 1) []

let posTr: P.t -> Nb.Z.t
= fun p0 ->
	let rec f: int -> P.t -> Nb.Z.t
	= fun shift -> function
	    | P.Coq_xI p ->
	      let bit = Nb.Z.shiftL Nb.Z.u shift in
	      let bits = f (shift + 1) p in
	      Nb.Z.orL bit bits
	    | P.Coq_xO p -> f (shift + 1) p
	    | P.Coq_xH -> Nb.Z.shiftL Nb.Z.u shift
	in
	f 0 p0

let posPr: P.t -> string
= fun p ->
	let z = posTr p in
	Nb.Z.pr z

let posPr': P.t -> char list
= fun p -> stringTr (posPr p)

let posPrRaw: P.t -> string
= fun p ->
	let rec pr l = function
	  | P.Coq_xI p -> pr ("xI"::l) p
	  | P.Coq_xO p -> pr ("xO"::l) p
	  | P.Coq_xH -> "xH"::l
	in
	String.concat " " (pr [] p)

let posPrRaw': P.t -> char list
= fun p -> stringTr (posPrRaw p)

let zTr: Z.t -> Nb.Z.t
= function
| Z.Z0 -> Nb.Z.z
| Z.Zpos p -> posTr p
| Z.Zneg p -> Nb.Z.neg (posTr p)

let zPr: Z.t -> string
= fun z -> Nb.Z.pr (zTr z)

let zPr': Z.t -> char list
= fun z -> stringTr (zPr z)

let zPrRaw: Z.t -> string
= function
| Z.Z0 -> "Z0"
| Z.Zpos p -> Printf.sprintf "Zpos (%s)" (posPrRaw p)
| Z.Zneg p -> Printf.sprintf "Zneg (%s)" (posPrRaw p)

let zPrRaw': Z.t -> char list
= fun z -> stringTr (zPrRaw z)
